package com.bernhardriedl.projects.doublefinder.logic;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import com.bernhardriedl.projects.tools.utils.ReadFile;
import com.bernhardriedl.projects.tools.utils.WriteFile;

public class DoubleFinder {

	private int counter=-1;

	/**
	 * Constructor
	 * @param fileName Filename
	 * @param createBackup boolean
	 * @throws Exception
	 */

	public DoubleFinder(String fileName, boolean createBackup) throws Exception {
		File fileToCheck=new File(fileName);

		if (!fileToCheck.exists()) {
			throw new Exception("File not found");
		}

		this.eliminateDoubles(fileToCheck, fileName, createBackup);
	}

	/**
	 * increments the double counter
	 */

	private void incrementCounter() {
		this.counter++;
	}

	/**
	 * resets the double counter
	 */

	private void resetCounter() {
		this.counter=0;
	}

	/**
	 * returns the removed double
	 * counter number
	 */

	public int getCounter() {
		return this.counter;
	}
    
	/**
	 * handle removal process
	 * @param fileToCheck File
	 * @param fileName String
	 * @param createBackup boolean
	 */

	private void eliminateDoubles(File fileToCheck, String fileName, boolean createBackup) throws IOException {
		if (createBackup) {
			File backupFile=new File(fileName+".bak");

			if (backupFile.exists())
				backupFile.delete();

				com.bernhardriedl.projects.tools.utils.Utils.copyFile(fileToCheck, backupFile);
		}

		ReadFile readFile=new ReadFile(fileName);
		readFile.open();

		Vector <String> lines = readFile.readLines();
		Vector <String> newLines = this.getDoubleFreeContent(lines);

		readFile.close();

		WriteFile writeFile=new WriteFile(fileName);
		writeFile.open();

		writeFile.overwriteFile(newLines);

		writeFile.close();
	}

	/**
	 * returns cleaned content
	 * @param lines Vector
	 * @return Vector
	 */

	private Vector <String> getDoubleFreeContent(Vector <String> lines) {
		this.resetCounter();

		Vector <String> newLines=new Vector <String> ();

		for (int i=0;i<lines.size();i++) {
			String line=lines.elementAt(i);

			if (line!=null) {
				newLines.addElement(line);

				/**
				 * loop through all elements
				 * after the actual element
				 */

				for (int j=i+1;j<lines.size();j++) {
					String toCheck = lines.elementAt(j);

					if (toCheck != null && toCheck.length()>0) {

						/**
						 * mark doubles with null values
						 */

						if (line.equals(lines.elementAt(j))) {
							lines.setElementAt(null, j);
							this.incrementCounter();
						}
					}
				}
			}
		}

		return newLines;
	}
}